@@ -149,6 +149,9 @@ class VariableInterface(object):
     def invalidate(self, force=False):
         pass
 
+    def is_console_set(self, varname):
+        pass
+
 
 class Variable(VariableInterface):
     """
@@ -172,6 +175,7 @@ class Variable(VariableInterface):
         Choice = "choice"
         Table = "table"
         Boolean = "bool"
+        Object = "object"
 
     # label of variable
     label = None
@@ -216,6 +220,10 @@ class Variable(VariableInterface):
 
     On = "on"
     Off = "off"
+    EmptyTable = [[]]
+
+    Readonly = READONLY
+    Writable = WRITEABLE
 
     def __init__(self, parent=None, **kwargs):
         """
@@ -268,6 +276,9 @@ class Variable(VariableInterface):
             return [[]]
         return ""
 
+    def is_console_set(self, varname):
+        return self.parent.is_console_set(varname)
+
     def GetBool(self, varname):
         return self.isTrue(self.Get(varname))
 
@@ -355,7 +366,6 @@ class Variable(VariableInterface):
         return val
 
     def _human(self):
-        # TODO: may be rest to NONE
         oldModeGet = self.modeGet
         try:
             self.modeGet = Variable.HUMAN
@@ -1232,6 +1242,7 @@ class DataVars(SimpleDataVars):
         self.defaultModule = "main"
         self.importedModules = []
         self.iniCache = {}
+        self.map_opt_vars = {}
 
     @contextmanager
     def useDefaultModule(self, modulename):
@@ -1757,8 +1768,18 @@ class DataVars(SimpleDataVars):
         for var in chain(normal, expert):
             varname = var.rpartition('.')[2]
             self.mapVarGroup[varname] = lastIndex
+            objVar = self.getInfo(var)
+            if objVar.opt is not None:
+                for opname in objVar.opt:
+                    self.map_opt_vars[opname] = varname
         return self
 
+    def is_console_set(self, varname):
+        for op in self.Get('main.cl_console_args'):
+            if self.map_opt_vars.get(op, '') == varname:
+                return True
+        return False
+
     def getGroups(self):
         """
         Get groups variables
