@@ -209,14 +209,13 @@ class SizeableDisk(object):
         if self.free_size < 0:
             needbytes = abs(self.free_size)
             availbytes = self.disk_size
-            avail = humanreadableSize(availbytes)
-            need = humanreadableSize(needbytes)
+            all_usebytes = availbytes + needbytes
+            all_use = humanreadableSize(all_usebytes)
             raise VirtualDiskError(
                 _("There is not enough space on this device "
                   "to perform the autopartition") + _(": ") +
-                _("{avail} available, need at {need} more").format(
-                    avail=avail,
-                    need=need))
+                _("need {need}").format(
+                    need=all_use))
 
 
 class VirtualDisk(SizeableDisk, DosDisk, GptDisk):
